using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using GrapeCity.Documents.Pdf.ViewerSupportApi.Controllers;
using GrapeCity.Documents.Pdf.ViewerSupportApi.Models;

namespace GcPdfViewerSupportApiDemo
{
    public class Startup
    {

        static Startup()
        {
#if DEBUG
            // VS's F5 sets the current working directory to project dir rather than bin dir.
            var exePath = new System.Uri(System.Reflection.Assembly.GetEntryAssembly().CodeBase).LocalPath;
            var directory = System.IO.Path.GetDirectoryName(exePath);
            System.IO.Directory.SetCurrentDirectory(directory);
#endif
            Licensing.AddLicenseKeys();
        }

        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
            var culture = configuration.GetValue("Culture", "");
            if (!string.IsNullOrEmpty(culture))
                GrapeCity.Documents.Pdf.ViewerSupportApi.Localization.Localizer.Culture = System.Globalization.CultureInfo.GetCultureInfo(culture);
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddCors(o => o.AddPolicy("WebCorsPolicy", builder =>
            {
                builder
                .AllowAnyMethod()
                .AllowAnyHeader()
                .SetIsOriginAllowed(origin => !string.IsNullOrEmpty(origin) && (origin.Contains(".grapecity") || origin.Contains("localhost") || origin.Contains("127.0.0.1")))
                .AllowCredentials();
            }));
            services.AddRazorPages();
            services.AddMvc((opts) => { opts.EnableEndpointRouting = false; });
            services.AddRouting();
            GrapeCity.Documents.Pdf.ViewerSupportApi.Connection.GcPdfViewerHub.ConfigureServices(services);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            app.UseCors("WebCorsPolicy");
            GrapeCity.Documents.Pdf.ViewerSupportApi.Connection.GcPdfViewerHub.Configure(app);
            GcPdfViewerController.Settings.VerifyToken += VerifyAuthToken;
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                app.UseExceptionHandler("/Error");
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }

            app.UseStaticFiles();

            app.UseRouting();
            
            app.UseAuthorization();


            app.UseMvcWithDefaultRoute();
        }

        private void VerifyAuthToken(object sender, VerifyTokenEventArgs e)
        {
            // Here is an example of how you can validate the authentication token
            // provided during viewer initialization - new GcPdfViewer(selector, { supportApi: { token: "support-api-demo" }});.
            string token = e.Token;
            if (string.IsNullOrEmpty(token) || !token.StartsWith("support-api-demo"))
            {
                e.Reject = true;
            }
        }
    }
}
